'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('elecpen');

function isFunction(v) {
  return typeof v == 'function';
}

function isDefined(v) {
  // filter `null` and `void 0`
  return v !== null;
}

function isDev() {
  return process.env['NODE_ENV'] !== 'production';
}

/**
 * Main method for create logger. The logger is just a function to recieve
 * the record and write to a stream.
 *
 * @param  {Writable}       writable    A writable stream for logging
 * @param  {string}         prefix      The prefix to write
 * @param  {string | bool}  date        The date format
 * @return {function} A function use like `console.log`
 */
function createLogger(writables, prefix, dateFormat) {
  if (!Array.isArray(writables)) writables = [writables];

  writables = writables.filter(function (w) {
    return isDefined(w) && isFunction(w.write);
  });

  var pre = '[' + prefix + ']';
  // default timestamp format
  if (dateFormat === true) dateFormat = 'YYYY-MM-DD HH:mm:ss';
  if (typeof dateFormat === 'string') {
    pre += ' ' + (0, _moment2.default)().format(dateFormat);
  }

  return function (msg) {
    writables.forEach(function (w) {
      return w.write(pre + ' ' + msg + '\n');
    });
  };
}

/**
 * Construct a default logger, it can record the verbose, info , warning, error message and
 * write it to the file you specified.
 *
 * Options:
 *     infoFile: Path of the file for recording verbose and info message.
 *     errFile: Path of the file for recording warning and error message.
 *     append: Decide append or truncate the file if it exists. Default to be true.
 *     timestamp: Represents the date format or pass true to use default format.
 *
 * @param {object} opts - options
 * @returns {object} The logger
 */
function useDefaultLogger(opts) {
  var infoFile = // string for a date format or pass true to use default format
  opts.infoFile;
  var errFile = opts.errFile;
  var _opts$append = opts.append;
  var append = _opts$append === undefined ? true : _opts$append;
  var _opts$timestamp = opts.timestamp;
  var timestamp = _opts$timestamp === undefined ? true : _opts$timestamp;

  // The value of `logToConsole` default to true in dev mode

  var logToConsole = opts.logToConsole;
  if (isDev() && !isDefined(logToConsole)) {
    logToConsole = true;
  }

  var flag = append ? 'a' : 'w';
  var infoStream = infoFile && _fs2.default.createWriteStream(infoFile, { flags: flag });
  var errStream = errFile && _fs2.default.createWriteStream(errFile, { flags: flag });

  var infoRecorder = streamRecorder();
  var errRecorder = streamRecorder();

  return {
    verbose: function verbose() {
      var stream = infoRecorder(infoFile, function (name) {
        return _fs2.default.createWriteStream(name, { flags: flag });
      });
      if (logToConsole) stream = [process.stdout, stream];

      createLogger(stream, 'Verbose', timestamp).apply(this, arguments);
    },
    info: function info() {
      var stream = infoRecorder(infoFile, function (name) {
        return _fs2.default.createWriteStream(name, { flags: flag });
      });
      if (logToConsole) stream = [process.stdout, stream];

      createLogger(stream, 'Info', timestamp).apply(this, arguments);
    },
    warning: function warning() {
      var stream = errRecorder(errFile, function (name) {
        return _fs2.default.createWriteStream(name, { flags: flag });
      });
      if (logToConsole) stream = [process.stderr, stream];

      createLogger(stream, 'Warning', timestamp).apply(this, arguments);
    },
    error: function error() {
      var stream = errRecorder(errFile, function (name) {
        return _fs2.default.createWriteStream(name, { flags: flag });
      });
      if (logToConsole) stream = [process.stderr, stream];

      createLogger(stream, 'Error', timestamp).apply(this, arguments);
    }
  };
}

/**
 * Record the stream by key
 * @return {function} A function return stream
 */
function streamRecorder() {
  var lastKey = undefined,
      lastStream = undefined;
  return function (key, createStream) {
    if (isFunction(key)) key = key.call();

    debug('Recorder get key: ' + key);

    if (key !== lastKey) {
      debug('Recorder will return a new stream');

      lastKey = key;
      lastStream = createStream.call(null, key);
    }

    return lastStream;
  };
}

createLogger.defaultLogger = useDefaultLogger;
createLogger.streamRecorder = streamRecorder;

module.exports = createLogger;